﻿using ServiceModelEx;
using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;

namespace Company.Infrastructure.Client
{
    public class AdapterProxy<I> : IDisposable where I : class
    {
        private I _Channel;

        public AdapterProxy()
        {
        }

        public AdapterProxy(Uri responseServiceUrl)
        {
            ResponseServiceMethodId = Guid.NewGuid().ToString();
            ResponseContext responseContext = new ResponseContext(responseServiceUrl.ToString(), ResponseServiceMethodId.ToString());

            // Add the response context to the incoming message header so that the static method ResponseContext.Current can find it on the header when it is called.
            GenericContext<ResponseContext> context = new GenericContext<ResponseContext>(responseContext);
            MessageHeader<GenericContext<ResponseContext>> genericHeader = new MessageHeader<GenericContext<ResponseContext>>(context);
            OperationContext.Current.IncomingMessageHeaders.Add(genericHeader.GetUntypedHeader(GenericContext<ResponseContext>.TypeName, GenericContext<ResponseContext>.TypeNamespace));
        }

        #region Properties

        protected I Channel
        {
            get
            {
                if (_Channel == null)
                {
                    _Channel = ProxyHelper.GetClientProxy<I>(this.GetType());
                }

                return _Channel;
            }
        }

        public string ResponseServiceMethodId { get; protected set; }

        #endregion

        ~AdapterProxy()
        {
            this.Dispose(false);
        }

        public bool ErrorRaised { get; protected set; }

        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool disposing)
        {
            if (disposing)
            {
                ICommunicationObject proxy = this.Channel as ICommunicationObject;

                try
                {
                    if (this.ErrorRaised)
                        proxy.Abort();
                    else
                        proxy.Close();
                }
                catch
                {
                    // If an exception was raised by the finally block and we are not already faulted then abort
                    // We do not want to call abort if we are faulted because .Abort would have already been 
                    // called and it is likely to be the source of the exception
                    if (!this.ErrorRaised)
                        proxy.Abort();
                }
            }
        }
    }
}