﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Configuration;
using System.ServiceModel.Description;
using Company.Infrastructure.Common;
using ServiceModelEx;

namespace Company.Infrastructure.Client
{
    public class PolicyChannelFactory<T> : ChannelFactory<T>
    {
        #region Private Members

        bool m_OverrideAvailable = false;

        #endregion

        #region Constructors

        public PolicyChannelFactory(Uri baseAddress, Binding applicationBinding, Uri responseServiceUrl)
            : this(applicationBinding, baseAddress)
        {

        }

        public PolicyChannelFactory(Binding binding, Uri baseAddress)
            : base(binding, AddressHelper.EnforceApplicationEndpointAddress(baseAddress, typeof(T)))
        {
            EnforcePolicy(Endpoint);
        }

        public PolicyChannelFactory(Binding binding, EndpointAddress endpointAddress)
            : base(binding, endpointAddress)
        {
            EnforcePolicy(Endpoint);
        }

        #endregion

        #region Methods

        // DESIGN NOTE:
        // As with the service-side, override ApplyConfiguration() to control and enforce configuration policy
        protected override void ApplyConfiguration(string configurationName)
        {
            m_OverrideAvailable = IsOverrideAvailable(Endpoint.Contract.ContractType);
            if (m_OverrideAvailable)
            {
                // DESIGN NOTE:
                // - Constrain override to binding only
                // - Attempt to load a single client section endpoint entry regardless of name by contract. If there is more than one, WCF will throw an exception.
                Endpoint.Binding = null;
                base.ApplyConfiguration("*");
            }
        }

        bool IsOverrideAvailable(Type contractType)
        {
            IEnumerable<ChannelEndpointElement> endpointsFromConfig = Settings.ClientEndpoints;
            return endpointsFromConfig.Any(endpoint => endpoint.Contract.Equals(contractType.FullName));
        }

        protected virtual bool EnforcePolicy(ServiceEndpoint configuredEndpoint)
        {
            // Enforce additional non-negotiable standard policy here.
            if (m_OverrideAvailable)
            {
                EnforcePolicyOnOverride(configuredEndpoint);
            }

            return m_OverrideAvailable;
        }

        void EnforcePolicyOnOverride(ServiceEndpoint endpoint)
        {
            // Enforce additional policy on override here.
            BindingHelper.ValidateBindingOverrideScheme(endpoint.Binding, endpoint.Address);
            endpoint.Binding = BindingHelper.EnforceBindingPolicyOnFileOverride(endpoint.Contract.Name, endpoint.Binding);
        }

        #endregion
    }
}