﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Company.Infrastructure.ServiceModel.CustomConfiguration;
using Company.Infrastructure.Common;
using System.Reflection;
using System.ServiceModel;

namespace Company.Infrastructure.Client
{
    public static class ProxyHelper
    {
        private static PolicyAssemblyType GetClientProxyPolicy(Type serviceType)
        {
            PolicyAssemblyType policy = new PolicyAssemblyType();

            var clientEntry = Settings.EnvironmentVariables.Clients.GetClientEntry(serviceType.ToString());
            string[] PolicyAssemblyArray = null;

            if (clientEntry != null)
            {
                OverrideKeyValueElement policyOverrideElement = clientEntry.GetEntry("ClientPolicy");

                if (policyOverrideElement != null)
                {
                    PolicyAssemblyArray = clientEntry.GetEntry("ClientPolicy").Value.Split(',');
                    policy.Assembly = PolicyAssemblyArray[0];
                    policy.Type = PolicyAssemblyArray[1];
                }
            }
            else
            {
                OverrideKeyValueElement environmentEntry = Settings.EnvironmentVariables.Overrides.GetEntry("ClientPolicy");

                if (environmentEntry != null)
                {
                    PolicyAssemblyArray = environmentEntry.Value.Split(',');
                    policy.Assembly = PolicyAssemblyArray[0];
                    policy.Type = PolicyAssemblyArray[1];
                }
            }

            if (string.IsNullOrWhiteSpace(policy.Type) || string.IsNullOrWhiteSpace(policy.Assembly))
            {
                return null;
            }

            return policy;
        }

        public static T GetClientProxy<T>(Type proxyType) where T : class
        {
            Type contractType = typeof(T);

            PolicyProxyAttribute policyBehaviorAttribute = null;

            // Load the policy to use for the service type from the config file.
            PolicyAssemblyType policyAssemblyInfo = GetClientProxyPolicy(contractType);

            Type proxyDefinition = null;
            if (policyAssemblyInfo == null)
            {
                IEnumerable<object> policyBehaviors = proxyType.GetCustomAttributes(typeof(PolicyProxyAttribute), false);

                if (policyBehaviors.Count() > 1)
                {
                    throw new InvalidOperationException(string.Format("More than one policy has been defined for [{0}]", proxyType.ToString()));
                }

                policyBehaviorAttribute = (PolicyProxyAttribute)policyBehaviors.FirstOrDefault();
            }
            else
            {                
                try
                {
                    // See if the policy type are looking for exists in the appdomain allready
                    proxyType = (from assembly in AppDomain.CurrentDomain.GetAssemblies()
                                 from atype in assembly.GetTypes()
                                 where atype.Name == policyAssemblyInfo.Type
                                 select atype).FirstOrDefault();
                }
                catch (Exception)
                {
                }

                // The types does not exist in the app domain. Load the specified assembly into the app domain.
                if (proxyType == null)
                {
                    Assembly policyAssembly = Assembly.Load(policyAssemblyInfo.Assembly);
                    proxyType = policyAssembly.GetType(policyAssemblyInfo.Type);
                }

                // The type exists in the appdomain, instantiate it.
                if (proxyType != null)
                {
                    proxyDefinition = proxyType.MakeGenericType(contractType);

                    // Request the policy behaviour to create the host.
                    policyBehaviorAttribute = (PolicyProxyAttribute)Activator.CreateInstance(proxyDefinition);
                }
            }

            if (policyBehaviorAttribute == null)
            {
                throw new InvalidOperationException(string.Format("No policy has been defined for [{0}]", contractType.ToString()));
            }

            EndpointAddress endpointAddress = Settings.EnvironmentVariables.GetClientEndpointAddress(proxyType);

            if (endpointAddress != null)
            {
                return policyBehaviorAttribute.CreateProxy<T>(endpointAddress).Channel as T;
            }
            else
            {
                Uri baseAddress = Settings.EnvironmentVariables.GetClientBaseAddress(contractType, policyBehaviorAttribute.GetBindingName(), Settings.ServerName);
                return policyBehaviorAttribute.CreateProxy<T>(baseAddress).Channel as T;
            }
        }
    }
}