﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;

namespace Company.Infrastructure.Common
{
    public static class AddressHelper
    {
        #region Public Methods

        public static Uri[] EnforceApplicationAddressing(Uri[] baseAddresses, Type serviceType)
        {
            List<Uri> newAddresses = new List<Uri>();

            foreach (Uri baseAddress in baseAddresses)
            {
                newAddresses.Add(EnforceApplicationAddress(baseAddress, serviceType));
            }

            return newAddresses.ToArray();
        }

        public static string EnforceEndpointAddress(Type contractType)
        {
            return contractType.FullName;
        }

        public static EndpointAddress EnforceApplicationEndpointAddress(Uri baseAddress, Type contractType)
        {
            return new EndpointAddress(EnforceApplicationAddress(baseAddress, contractType).AbsoluteUri + EnforceEndpointAddress(contractType));
        }

        public static Uri[] EnforceQueueAddressing(Uri[] baseAddresses, Type serviceType)
        {
            List<Uri> newAddresses = new List<Uri>();

            foreach (Uri baseAddress in baseAddresses)
            {
                newAddresses.Add(EnforceQueueAddress(baseAddress, serviceType));
            }

            return newAddresses.ToArray();
        }

        public static Type GetLastContract(Type serviceType)
        {
            Type type = GetInterfaces(serviceType).LastOrDefault();

            return type;
        }

        public static Type GetPrimaryContract(Type serviceType)
        {
            Type type = GetInterfaces(serviceType).FirstOrDefault();

            return type;
        }

        #endregion

        #region Private Methods

        private static Uri EnforceApplicationAddress(Uri baseAddress, Type namespaceType)
        {
            string applicationAddress = baseAddress.AbsoluteUri + (baseAddress.AbsoluteUri.EndsWith("/") ? "" : "/");

            if (!baseAddress.AbsoluteUri.Contains(SubsystemName(namespaceType)))
            {
                applicationAddress += SubsystemName(namespaceType) + "/";
            }

            return new Uri(applicationAddress);
        }

        private static Uri EnforceQueueAddress(Uri baseAddress, Type namespaceType)
        {
            string applicationAddress = baseAddress.AbsoluteUri + (baseAddress.AbsoluteUri.EndsWith("/") ? "" : "/");

            applicationAddress = string.Format("{0}private/{1}", applicationAddress, SubsystemName(namespaceType));

            return new Uri(applicationAddress);
        }

        private static Type[] GetInterfaces(Type serviceType)
        {
            Type[] interfaces = (from a in serviceType.GetInterfaces()
                                 where a.GetCustomAttributes(typeof(ServiceContractAttribute), false).Any()
                                 select a).ToArray();

            return interfaces;
        }

        static string SubsystemName(Type namespaceType)
        {
            string[] elements = namespaceType.Namespace.Split('.');

            return elements[elements.Length - 1];
        }

        #endregion
    }
}