﻿using Company.Infrastructure.Security;
using System;
using System.Diagnostics;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace Company.Infrastructure.Common
{
    public static class BindingHelper
    {
        public static TimeSpan DefaultConnectivityTimeout
        {
            get
            {
                return new TimeSpan(0, 0, 15);
            }
        }

        public static TimeSpan DefaultDebugTimeout
        {
            get
            {
                return new TimeSpan(0, 0, 30);
            }
        }

        public static long DefaultMaxMessageSize
        {
            get
            {
                return (1024 * 1024);
            }
        }

        public static void EnforceTimeoutPolicy(Binding binding, TimeSpan connectiviyTimeout, TimeSpan debugTimeout)
        {
            Debug.Assert(connectiviyTimeout != TimeSpan.Zero);
            Debug.Assert(debugTimeout != TimeSpan.Zero);

            if (Debugger.IsAttached)
            {
                binding.ReceiveTimeout = debugTimeout;
                binding.SendTimeout = debugTimeout;
            }
            else
            {
                binding.ReceiveTimeout = connectiviyTimeout;
                binding.SendTimeout = connectiviyTimeout;
            }
        }

        public static void ValidateBindingOverrideScheme(Binding bindingOverride, EndpointAddress applicationAddress)
        {
            Debug.Assert(bindingOverride != null);
            Debug.Assert(applicationAddress != null);
            Debug.Assert(applicationAddress.Uri != null);

            if (!bindingOverride.Scheme.Equals(applicationAddress.Uri.Scheme))
            {
                throw new InvalidOperationException(string.Format("Binding override {0} does not match application {1}.", bindingOverride.Scheme, applicationAddress.Uri.Scheme));
            }
        }

        public static Binding EnforceBindingPolicyOnFileOverride(Binding binding)
        {
            return EnforceBindingPolicyOnFileOverride(string.Empty, binding);
        }

        public static Binding EnforceBindingPolicyOnFileOverride(string contractName, Binding binding)
        {
            //if (contractName.Contains("Mtom"))
            //{
            //    // BindingHelper.Intranet.EnforceMtomBindingPolicy(...);
            //}
            //else if (contractName.Contains("Streamed"))
            //{
            //    // BindingHelper.Intranet.EnforceStreamedBindingPolicy(...);
            //}
            //else if (contractName.Contains("Bus"))
            //{
            //    // BindingHelper.Bus.EnforceBindingPolicy(...);
            //}
            //else if (binding is CustomBinding)
            //{
            //    // BindingHelper.Custom.EnforceBindingPolicy(binding as CustomBinding);
            //}
            //else if (BindingHelper.Bus.IsBusBinding(binding))
            //{
            //    // BindingHelper.Bus.EnforceBindingPolicy(binding);
            //}
            //else if (binding is NetTcpBinding)
            //{
            //    NetTcpBinding tcpBinding = binding as NetTcpBinding;
            //    BindingHelper.Intranet.EnforceBindingPolicy(tcpBinding, tcpBinding.MaxReceivedMessageSize, tcpBinding.SendTimeout, DefaultDebugTimeout);
            //}
            //else
            //{
            //    throw new InvalidOperationException(binding.Scheme + " is invalid for override.");
            //}
            //return binding;

            return null;
        }

        //public static class Intranet
        //{
        //    static NetTcpBinding m_BindingOverride;
        //    static NetTcpBinding DefaultBinding()
        //    {
        //        NetTcpBinding binding = new NetTcpBinding(SecurityMode.None, true);
        //        return SecurityBindingProvider.SecureBinding(binding) as NetTcpBinding;
        //    }

        //    internal static void EnforceBindingPolicy(NetTcpBinding tcpBinding, long maxMessageSize, TimeSpan connectiviyTimeout, TimeSpan debugTimeout)
        //    {
        //        if (tcpBinding == null)
        //        {
        //            throw new ArgumentNullException("tcpBinding");
        //        }

        //        tcpBinding.ReliableSession.Enabled = true;
        //        tcpBinding.PortSharingEnabled = true;
        //        if (maxMessageSize > 0)
        //        {
        //            tcpBinding.MaxReceivedMessageSize = maxMessageSize;
        //        }
        //        if (tcpBinding.TransferMode == TransferMode.Streamed)
        //        {
        //            Debug.Assert(tcpBinding.ReliableSession.Enabled == false, "Reliable session is not supported when transfermode is set to streamed.");
        //            tcpBinding.ReliableSession.Enabled = false;
        //        }
        //        else
        //        {
        //            Debug.Assert(maxMessageSize <= int.MaxValue, string.Format("The MaxMessageSize ({0}) provided is too large. When not using streaming, MaxMessageSize must be less than 2GB.", maxMessageSize));
        //        }
        //        EnforceTimeoutPolicy(tcpBinding, connectiviyTimeout, debugTimeout);
        //    }

        //    // DESIGN NOTE:
        //    // Provide for manual override.
        //    public static void SetBindingOverride(NetTcpBinding binding)
        //    {
        //        m_BindingOverride = binding;
        //    }

        //    public static NetTcpBinding Binding()
        //    {
        //        return Binding(DefaultMaxMessageSize, DefaultConnectivityTimeout, DefaultDebugTimeout);
        //    }

        //    public static NetTcpBinding Binding(long maxMessageSize, TimeSpan connectiviyTimeout, TimeSpan debugTimeout)
        //    {
        //        NetTcpBinding binding = m_BindingOverride;
        //        try
        //        {
        //            binding = new NetTcpBinding("IntranetOverride");
        //            binding = EnforceBindingPolicyOnFileOverride(binding) as NetTcpBinding;
        //        }
        //        catch
        //        {
        //            if (binding == null)
        //            {
        //                binding = DefaultBinding();
        //            }
        //            EnforceBindingPolicy(binding, maxMessageSize, connectiviyTimeout, debugTimeout);
        //        }

        //        return binding;
        //    }
        //}

        //public static class Component
        //{
        //    static NetNamedPipeBinding m_BindingOverride;
        //    static NetNamedPipeBinding DefaultBinding()
        //    {
        //        return new NetNamedPipeBinding(NetNamedPipeSecurityMode.Transport);
        //    }

        //    internal static void EnforceBindingPolicy(NetNamedPipeBinding namedPipeBinding, long maxMessageSize, TimeSpan connectiviyTimeout, TimeSpan debugTimeout)
        //    {
        //        if (namedPipeBinding == null)
        //        {
        //            throw new ArgumentNullException("namedPipeBinding");
        //        }

        //        namedPipeBinding.TransactionFlow = true;

        //        if (maxMessageSize > 0)
        //        {
        //            namedPipeBinding.MaxReceivedMessageSize = maxMessageSize;
        //        }
        //        if (namedPipeBinding.TransferMode == TransferMode.Streamed)
        //        {
        //            Debug.Assert(maxMessageSize <= int.MaxValue, string.Format("The MaxMessageSize ({0}) provided is too large. When not using streaming, MaxMessageSize must be less than 2GB.", maxMessageSize));
        //        }
        //        EnforceTimeoutPolicy(namedPipeBinding, connectiviyTimeout, debugTimeout);
        //    }

        //    public static void SetBindingOverride(NetNamedPipeBinding binding)
        //    {
        //        m_BindingOverride = binding;
        //    }

        //    public static NetNamedPipeBinding Binding()
        //    {
        //        return Binding(DefaultMaxMessageSize, DefaultConnectivityTimeout, DefaultDebugTimeout);
        //    }

        //    public static NetNamedPipeBinding Binding(long maxMessageSize, TimeSpan connectiviyTimeout, TimeSpan debugTimeout)
        //    {
        //        NetNamedPipeBinding binding = m_BindingOverride;
        //        try
        //        {
        //            binding = new NetNamedPipeBinding("ComponentOverride");
        //            //binding = EnforceBindingPolicyOnFileOverride(binding) as NetNamedPipeBinding;
        //        }
        //        catch
        //        {
        //            if (binding == null)
        //            {
        //                binding = DefaultBinding();
        //            }
        //            EnforceBindingPolicy(binding, maxMessageSize, connectiviyTimeout, debugTimeout);
        //        }

        //        return binding;
        //    }
        //}

        public static class Bus
        {
            public static bool IsBusBinding(Binding binding)
            {
                return binding is NetMsmqBinding;
            }
        }
    }
}