﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace Assima.Infrastructure.Common
{
    public static class CertificateHelper
    {
        /// <summary>
        /// Get the certificate.
        /// </summary>
        /// <param name="subjectName">The encryption certificate subject name.</param>
        /// <returns>A X509Certificate2 certificate.</returns>
        public static X509Certificate2 GetCertificate(string subjectName)
        {
            var store = new X509Store(StoreName.My, StoreLocation.LocalMachine);
            X509Certificate2Collection certificates = null;
            store.Open(OpenFlags.ReadOnly);

            try
            {
                certificates = store.Certificates;
                var certs = certificates.OfType<X509Certificate2>().Where(x => x.SubjectName.Name.Equals(subjectName, StringComparison.OrdinalIgnoreCase)).ToList();

                if (certs.Count == 0)
                    throw new SecurityException(string.Format("No certificate was found for subject Name {0}", subjectName));
                else if (certs.Count > 1)
                    throw new SecurityException(string.Format("There are multiple certificates for subject Name {0}", subjectName));

                return new X509Certificate2(certs[0]);
            }
            finally
            {
                if (certificates != null)
                {
                    for (var i = 0; i < certificates.Count; i++)
                        certificates[i].Reset();
                }

                store.Close();
            }
        }
    }
}