﻿using Company.Infrastructure.ServiceModel.CustomConfiguration;
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration;
using System.Linq;
using System.ServiceModel.Configuration;
using System.Web.Configuration;

namespace Company.Infrastructure.Common
{
    public static class Settings
    {
        private static EnvironmentConfig _EnvironmentVariables;

        public static EnvironmentConfig EnvironmentVariables
        {
            get { return _EnvironmentVariables; }
        }

        static NameValueCollection m_AppSettings;

        static NameValueCollection GetAppSettings
        {
            get
            {
                return m_AppSettings ??
                       (m_AppSettings = IsWebConfiguration() ?
                                        WebConfigurationManager.AppSettings :
                                        ConfigurationManager.AppSettings);
            }
        }

        static object GetSection(string sectionPath)
        {
            if (sectionPath == null)
                throw new ArgumentNullException("sectionPath");

            return IsWebConfiguration() ?
                   WebConfigurationManager.GetSection(sectionPath) :
                   ConfigurationManager.GetSection(sectionPath);
        }

        static string MachineName
        {
            get
            {
                return System.Diagnostics.Process.GetCurrentProcess().MachineName;
            }
        }

        static string ProcessName
        {
            get
            {
                return System.Diagnostics.Process.GetCurrentProcess().ProcessName;
            }
        }

        public static bool IsWebConfiguration()
        {
            return ProcessName.ToLower().Contains("w3wp") || ProcessName.ToLower().Contains("webdev.webserver");
        }

        public static Configuration GetConfiguration()
        {
            return Settings.IsWebConfiguration() ?
                   WebConfigurationManager.OpenWebConfiguration(null) :
                   ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
        }

        public static Uri ComponentApplicationAddress
        {
            get
            {
                return new UriBuilder(Uri.UriSchemeNetPipe, ServerName, -1, ProcessName).Uri;
            }
        }

        public static IEnumerable<ChannelEndpointElement> ClientEndpoints
        {
            get
            {
                ClientSection section = GetSection("system.serviceModel/client") as ClientSection;
                return section == null ? new ChannelEndpointElement[0] : section.Endpoints.OfType<ChannelEndpointElement>();
            }
        }

        public static string ServerName
        {
            get
            {
                string serverName = "localhost";
                if (EnvironmentVariables.Overrides.GetEntry("Host") != null)
                {
                    serverName = EnvironmentVariables.Overrides.GetEntry("Host").Value;
                }
                return serverName;
            }
        }

        public static IEnumerable<ServiceElement> Services
        {
            get
            {
                ServicesSection section = GetSection("system.serviceModel/services") as ServicesSection;
                return section == null ? new ServiceElement[0] : section.Services.OfType<ServiceElement>();
            }
        }

        static Settings()
        {
            _EnvironmentVariables = (EnvironmentConfig)GetConfiguration().GetSection("CompanyServicesConfiguration/EnvironmentConfig");
            if (_EnvironmentVariables == null) _EnvironmentVariables = new EnvironmentConfig();
        }
    }
}