﻿using System;
using System.Collections.Generic;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using Company.Infrastructure.Client;
using Company.Infrastructure.Security;
using System.ServiceModel;

namespace Company.Infrastructure.Policy.Intranet.Client
{
    public class IntranetPolicyChannelFactory<T> : PolicyChannelFactory<T>
    {
        #region Constructor

        public IntranetPolicyChannelFactory(Binding binding, Uri baseAddress)
            : base(binding, baseAddress)
        {
        }

        public IntranetPolicyChannelFactory(Binding binding, EndpointAddress endpointAddress)
            : base(binding, endpointAddress)
        {
        }

        #endregion

        #region Override Methods

        protected override bool EnforcePolicy(ServiceEndpoint configuredEndpoint)
        {
            bool isOverrideAvailable = base.EnforcePolicy(configuredEndpoint);

            if (!isOverrideAvailable)
            {
                this.EnforceSecurityBehaviorPolicy(configuredEndpoint);
            }

            return isOverrideAvailable;
        }

        #endregion

        #region Private Methods

        private void EnforceSecurityBehaviorPolicy(ServiceEndpoint configuredEndpoint)
        {
            IEnumerable<IEndpointBehavior> securityBehaviors = SecurityBehaviorProvider.CreateEndPointBehaviors(this.GetType());

            foreach (IEndpointBehavior behavior in securityBehaviors)
            {
                if (configuredEndpoint.EndpointBehaviors.Contains(behavior.GetType()))
                {
                    configuredEndpoint.EndpointBehaviors.Remove(behavior.GetType());
                }

                configuredEndpoint.EndpointBehaviors.Add(behavior);
            }
        }

        #endregion
    }
}