﻿using System;
using System.ServiceModel.Channels;
using Company.Infrastructure.Client;
using System.ServiceModel;

namespace Company.Infrastructure.Policy.Intranet.Client
{
    public class IntranetProxy<I> : PolicyProxyBase<I> where I : class
    {
        private EndpointAddress _endpointAddress = null;

        public IntranetProxy(Uri baseAddress)
        {
            Initialize(baseAddress);
        }

        public IntranetProxy(EndpointAddress endpointAddress)
        {
            _endpointAddress = endpointAddress;
            Initialize(endpointAddress.Uri);
        }

        protected override Binding Binding(long maxMessageSize, TimeSpan connectivityTimeout, TimeSpan debugTimeout)
        {
            return IntranetBindingHelper.Binding(maxMessageSize, connectivityTimeout, debugTimeout);
        }

        protected override PolicyChannelFactory<I> CreatePolicyChannelFactory(Binding binding, Uri baseAddress)
        {

            if (_endpointAddress == null)
            {
                return new IntranetPolicyChannelFactory<I>(binding, baseAddress);
            }

            return new IntranetPolicyChannelFactory<I>(binding, _endpointAddress);
        }
    }
}