﻿using System;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Transactions;
using Company.Infrastructure.Server;

namespace Company.Infrastructure.Policy.Intranet.Server
{
    public class IntranetPolicy : PolicyServiceBehaviorBase
    {
        #region Constructor

        public IntranetPolicy()
        {
            OperationBehaviors.Add(typeof(Microsoft.Practices.EnterpriseLibrary.Validation.Integration.WCF.ValidationBehavior));

            // Setting defaults
            this.InstanceContextMode = System.ServiceModel.InstanceContextMode.PerCall;
            this.TransactionIsolationLevel = IsolationLevel.ReadCommitted;
            this.TransactionTimeout = "00:01:00";
        }

        #endregion

        #region Properties

        public InstanceContextMode InstanceContextMode { get; set; }

        public string Namespace { get; set; }

        public IsolationLevel TransactionIsolationLevel { get; set; }

        public string TransactionTimeout { get; set; }

        #endregion

        #region Override Methods

        public override string GetBindingName()
        {
            return "netTcpBinding";
        }

        public override ServiceHost CreateHost(Type serviceType, System.Uri[] baseAddresses)
        {
            Type hostDefinition = typeof(IntranetServiceHost<>).MakeGenericType(serviceType);
            ServiceHost host = (ServiceHost)Activator.CreateInstance(hostDefinition, baseAddresses);

            return host;
        }

        protected override void EnforceServiceBehavior(ServiceDescription serviceDescription, ServiceHostBase serviceHostBase)
        {
            ServiceBehaviorAttribute serviceBehavior = (ServiceBehaviorAttribute)serviceDescription.Behaviors.FirstOrDefault(behavior => behavior is ServiceBehaviorAttribute);
            if (serviceBehavior != null)
            {
                serviceBehavior.InstanceContextMode = this.InstanceContextMode;
                serviceBehavior.Namespace = this.Namespace;
                serviceBehavior.TransactionIsolationLevel = this.TransactionIsolationLevel;
                serviceBehavior.TransactionTimeout = this.TransactionTimeout;
            }
        }

        #endregion
    }
}