﻿using System;
using System.Collections.Generic;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using Company.Infrastructure.Common;
using Company.Infrastructure.Security;
using Company.Infrastructure.Server;

namespace Company.Infrastructure.Policy.Intranet.Server
{
    public class IntranetServiceHost<T> : PolicyServiceHostBase<T>
    {
        #region Constructors

        public IntranetServiceHost(params Uri[] baseAddresses)
            : base(GetEndpointAddress(baseAddresses))
        {
        }

        #endregion

        #region Override Methods

        protected override Binding Binding(Type contractType)
        {
            return IntranetBindingHelper.Binding(DefaultMaxMessageSize, DefaultConnectivityTimeout, DefaultDebugTimeout);
        }

        protected override void EnforceBehaviorPolicy(ServiceDescription description)
        {
        }

        protected override void EnforceSecurityBehaviorPolicy(ServiceDescription description)
        {
            IEnumerable<IServiceBehavior> securityBehaviors = SecurityBehaviorProvider.CreateServiceBehaviors(this.GetType());

            foreach (IServiceBehavior behavior in securityBehaviors)
            {
                description.Behaviors.Add(behavior);
            }
        }

        protected override void EnforceEndPointPolicy(ServiceEndpoint endpoint)
        {
        }

        protected override void EnforceEndPointSecurityPolicy(ServiceEndpoint endpoint)
        {
        }

        protected static Uri[] GetEndpointAddress(Uri[] baseAddresses)
        {
            return AddressHelper.EnforceApplicationAddressing(baseAddresses, AddressHelper.GetPrimaryContract(typeof(T)));
        }

        #endregion
    }
}