﻿using System;
using System.Diagnostics;
using System.ServiceModel;
using Company.Infrastructure.Common;
using Company.Infrastructure.Security;

namespace Company.Infrastructure.Policy.Intranet
{
    public static class IntranetBindingHelper
    {
        static NetTcpBinding m_BindingOverride;
        static NetTcpBinding DefaultBinding()
        {
            NetTcpBinding binding = new NetTcpBinding(SecurityMode.None, true);
            return SecurityBindingProvider.SecureBinding(binding) as NetTcpBinding;
        }

        internal static void EnforceBindingPolicy(NetTcpBinding tcpBinding, long maxMessageSize, TimeSpan connectiviyTimeout, TimeSpan debugTimeout)
        {
            if (tcpBinding == null)
            {
                throw new ArgumentNullException("tcpBinding");
            }

            tcpBinding.ReliableSession.Enabled = true;
            tcpBinding.PortSharingEnabled = true;
            if (maxMessageSize > 0)
            {
                tcpBinding.MaxReceivedMessageSize = maxMessageSize;
            }
            if (tcpBinding.TransferMode == TransferMode.Streamed)
            {
                Debug.Assert(tcpBinding.ReliableSession.Enabled == false, "Reliable session is not supported when transfermode is set to streamed.");
                tcpBinding.ReliableSession.Enabled = false;
            }
            else
            {
                Debug.Assert(maxMessageSize <= int.MaxValue, string.Format("The MaxMessageSize ({0}) provided is too large. When not using streaming, MaxMessageSize must be less than 2GB.", maxMessageSize));
            }
            BindingHelper.EnforceTimeoutPolicy(tcpBinding, connectiviyTimeout, debugTimeout);
        }

        // DESIGN NOTE:
        // Provide for manual override.
        public static void SetBindingOverride(NetTcpBinding binding)
        {
            m_BindingOverride = binding;
        }

        public static NetTcpBinding Binding()
        {
            return Binding(BindingHelper.DefaultMaxMessageSize, BindingHelper.DefaultConnectivityTimeout, BindingHelper.DefaultDebugTimeout);
        }

        public static NetTcpBinding Binding(long maxMessageSize, TimeSpan connectiviyTimeout, TimeSpan debugTimeout)
        {
            NetTcpBinding binding = m_BindingOverride;
            try
            {
                binding = new NetTcpBinding("IntranetOverride");
                binding = BindingHelper.EnforceBindingPolicyOnFileOverride(binding) as NetTcpBinding;
            }
            catch
            {
                if (binding == null)
                {
                    binding = DefaultBinding();
                }
                EnforceBindingPolicy(binding, maxMessageSize, connectiviyTimeout, debugTimeout);
            }

            return binding;
        }
    }
}