﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Security;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;
using System.Threading.Tasks;

namespace Company.Infrastructure.Security
{
    public static class SecurityBindingProvider
    {
        private static NetTcpBinding ApplySecurityConfiguration(NetTcpBinding binding, SecurityType securityType)
        {
            if (securityType != SecurityType.None)
            {
                binding.Security.Mode = SecurityMode.Transport;
                binding.Security.Transport.ProtectionLevel = ProtectionLevel.EncryptAndSign;
                binding.Security.Transport.ClientCredentialType = TcpClientCredentialType.Windows;
                binding.Security.Message.ClientCredentialType = MessageCredentialType.None;
            }
            return binding;
        }

        private static NetMsmqBinding ApplySecurityConfiguration(NetMsmqBinding binding, SecurityType securityType)
        {
            if (securityType != SecurityType.None)
            {
                binding.Security.Mode = NetMsmqSecurityMode.None;
                binding.Security.Transport.MsmqProtectionLevel = ProtectionLevel.None;
                binding.Security.Transport.MsmqAuthenticationMode = MsmqAuthenticationMode.WindowsDomain;
                binding.Security.Message.ClientCredentialType = MessageCredentialType.None;
            }

            return binding;
        }

        public static Binding SecureBinding(Binding binding)
        {
            SecurityType securityType = SecuritySettings.GetSecurityType();

            if (binding is NetTcpBinding)
            {
                binding = ApplySecurityConfiguration(binding as NetTcpBinding, securityType);
            }

            if (binding is NetMsmqBinding)
            {
                binding = ApplySecurityConfiguration(binding as NetMsmqBinding, securityType);
            }

            return binding;
        }

    }
}
