﻿using Company.Infrastructure.ServiceModel.CustomConfiguration;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Company.Infrastructure.Security
{
    public static class SecuritySettings
    {
        private static EnvironmentConfig _EnvironmentVariables;

        public static EnvironmentConfig EnvironmentVariables
        {
            get { return _EnvironmentVariables; }
        }

        private static Configuration GetConfiguration()
        {
            return ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None);
        }

        private static SecurityType StringToSecurityType(string securityTypeString)
        {
            if (securityTypeString == null) return SecurityType.Impersonation;

            switch (securityTypeString.ToLower())
            {
                case "none":
                    return SecurityType.None;
                case "impersonation":
                    return SecurityType.Impersonation;
                case "delegation":
                    return SecurityType.Delegation;
                default: return SecurityType.Impersonation;
            }
        }

        public static SecurityType GetSecurityType()
        {
            String securityTypeString = "None";

            if (_EnvironmentVariables != null)
            {
                OverrideKeyValueElement securityTypeOverride = _EnvironmentVariables.Overrides.GetEntry("SecurityType");
                securityTypeString = securityTypeOverride != null ? securityTypeOverride.Value : "None";
            }

            SecurityType securityType = StringToSecurityType(securityTypeString);

            return securityType;
        }

        static SecuritySettings()
        {
            _EnvironmentVariables = (EnvironmentConfig)GetConfiguration().GetSection("CompanyServicesConfiguration/EnvironmentConfig");
        }        
    }
}
