﻿using System;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;

namespace Company.Infrastructure.Server
{
    [AttributeUsage(AttributeTargets.Method, AllowMultiple = false, Inherited = false)]
    public abstract class PolicyOperationBehaviorBase : Attribute, IOperationBehavior
    {
        protected Type OperationInvoker
        { get; set; }

        public void AddBindingParameters(OperationDescription operationDescription, BindingParameterCollection bindingParameters)
        { }

        public void ApplyClientBehavior(OperationDescription operationDescription, ClientOperation clientOperation)
        { }

        public void ApplyDispatchBehavior(OperationDescription operationDescription, DispatchOperation dispatchOperation)
        {
            InstallCustomInvoker(operationDescription, dispatchOperation);
        }

        public void Validate(OperationDescription operationDescription)
        { }

        protected virtual void InstallCustomInvoker(OperationDescription operationDescription, DispatchOperation dispatchOperation)
        {
            if (OperationInvoker != null)
            {
                dispatchOperation.Invoker = (IOperationInvoker)Activator.CreateInstance(OperationInvoker, dispatchOperation.Invoker);
            }
        }
    }
}