﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceModel.Discovery;
using System.ServiceProcess;

namespace Company.Infrastructure.ServiceModel.Hosts
{
    /// <summary>
    /// Abstract class to be used when wanting to host multiple WCF services within a windows service process.
    /// </summary>
    /// <remarks>InstallUtil asselmblyname.exe InstallUtil /u asselmblyname.exe</remarks>
    public abstract class WindowsServiceWCFHostManager : ServiceBase
    {
        private ServiceHostManager serviceManager; 

        /// <summary>
        /// Abstract property to the windows service name that must be implemented.
        /// </summary>
        public abstract string WindowsServiceName { get; }

        /// <summary>
        /// Provides a means of starting the service manually when its not running as service.
        /// </summary>
        public void Start()
        {            
            serviceManager.Hosts.AddRange(RegisterServiceHosts());
            serviceManager.OpenHosts();            
        }

        /// <summary>
        /// Instantiates a instance of <see cref="WindowsServiceWCFHostManager"/>
        /// </summary>
        protected WindowsServiceWCFHostManager()
        {
            this.ServiceName = this.WindowsServiceName;
            serviceManager = new ServiceHostManager(this.ServiceName);
        }

        public abstract IEnumerable<ServiceHost> RegisterServiceHosts();

        /// <summary>
        /// Starts all the services hosted wihtin the process.
        /// </summary>
        /// <param name="args"></param>
        protected override void OnStart(string[] args)
        {
            Start();
        }

        /// <summary>
        /// Stops all the services that are hosted wihtin the process.
        /// </summary>
        protected override void OnStop()
        {
            serviceManager.CloseHosts();
        }
    }
}
