﻿// © 2014 IDesign Inc. All rights reserved 
//Questions? Comments? go to 
//http://www.idesign.net

using System.ServiceModel;
using System.ServiceModel.Channels;

namespace ServiceModelEx
{
    public class HeaderChannelFactory<T, H> : InterceptorChannelFactory<T> where T : class
    {
        public H Header
        { get; protected set; }

        public HeaderChannelFactory()
            : this(default(H))
        { }

        public HeaderChannelFactory(string endpointName)
            : this(default(H), endpointName)
        { }

        public HeaderChannelFactory(Binding binding, EndpointAddress remoteAddress)
            : this(default(H), binding, remoteAddress)
        { }

        public HeaderChannelFactory(H header)
        {
            Header = header;
        }

        public HeaderChannelFactory(H header, string endpointName)
            : base(endpointName)
        {
            Header = header;
        }

        public HeaderChannelFactory(H header, Binding binding, EndpointAddress remoteAddress)
            : base(binding, remoteAddress)
        {
            Header = header;
        }
        protected override void PreInvoke(ref Message request)
        {
            GenericContext<H> context = new GenericContext<H>(Header);
            MessageHeader<GenericContext<H>> genericHeader = new MessageHeader<GenericContext<H>>(context);
            request.Headers.Add(genericHeader.GetUntypedHeader(GenericContext<H>.TypeName, GenericContext<H>.TypeNamespace));

            // DP 2015-06-05 Add the callstack info to the header for InProc factory
            AddCallstack(ref request);
        }

        private void AddCallstack(ref Message request)
        {
            SecurityCallStack callstack = SecurityCallStackContext.Current;

            if (callstack == null)
            {
                // Create a new callstack
                callstack = new SecurityCallStack();
            }
            else
            {
                // Append to callstack
                callstack.AppendCall();
            }

            // Package callstack in the header
            MessageHeader<GenericContext<SecurityCallStack>> genericSecurityHeader = new MessageHeader<GenericContext<SecurityCallStack>>(GenericContext<SecurityCallStack>.Current);
            MessageHeader header = genericSecurityHeader.GetUntypedHeader(GenericContext<SecurityCallStack>.TypeName, GenericContext<SecurityCallStack>.TypeNamespace);

            request.Headers.Add(header);
        }
    }
}