﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.Security.Claims;

namespace ServiceModelEx.Security.Principal
{
    public class IdentityUserData
    {
        private ClaimsIdentity _identity = null;
        public IdentityUserData()
        {
            _identity = IdentityUserData.FindIdentity();

        }

        public static void AddIdentity(IEnumerable<Claim> claims)
        {
            if (claims != null)
            {
                ClaimsIdentity currentIdentity = IdentityUserData.FindIdentity();
                if (currentIdentity != null)
                {
                    Claim currentClaim = currentIdentity.Claims.FirstOrDefault();
                    while (currentClaim != null)
                    {
                        currentIdentity.RemoveClaim(currentClaim);
                        currentClaim = currentIdentity.Claims.FirstOrDefault();
                    }                   
                }

                ClaimsIdentity identity = new ClaimsIdentity(claims);
                ClaimsPrincipal.Current.AddIdentity(identity);
            }
        }

        public static ClaimsIdentity FindIdentity()
        {            
            if (ClaimsPrincipal.Current != null && ClaimsPrincipal.Current.Identities.Any())
            {
                IEnumerable<ClaimsIdentity> CompanyIdentities = new List<ClaimsIdentity>();
                CompanyIdentities = ClaimsPrincipal.Current.Identities.Select(i => i).Where(ii => ii.Claims.Any(c => c.Type.StartsWith(CustomClaimTypes.ClaimTypesBase)));
                return CompanyIdentities.LastOrDefault();
            }
            return null;
        }
        
        /// <summary>
        /// Gets a GUID that uniquely identifies a call chain
        /// </summary>
        public Guid ActivityId
        {
            get
            {
                return ClaimsBuilder.GetClaimValueGuid(this._identity, CustomClaimTypes.ActivityId);
            }
        }

        /// <summary>
        /// Gets the authentication method.
        /// </summary>
        public string AuthenticationMethod
        {
            get
            {
                return ClaimsBuilder.GetClaimValue(this._identity, ClaimTypes.AuthenticationMethod);                
            }
        }

        /// <summary>
        /// Gets the source address of the caller.
        /// </summary>
        public string CallerAddress
        {
            get
            {
                return ClaimsBuilder.GetClaimValue(this._identity, CustomClaimTypes.CallerAddress);                
            }
        }

        /// <summary>
        /// Gets a descriptive string identifying the type of the caller.
        /// </summary>
        public string CallerType
        {
            get
            {
                return ClaimsBuilder.GetClaimValue(this._identity, CustomClaimTypes.CallerType);
            }
        }

        /// <summary>
        /// Gets the Email of the user.
        /// </summary>
        /// <value>
        /// The Email of the user.
        /// </value>
        public string Email
        {
            get
            {
                return ClaimsBuilder.GetClaimValue(this._identity, ClaimTypes.Email);
            }
        }

        /// <summary>
        /// Gets the windows name of the account that is executing the code.
        /// </summary>
        public string ExecutingWindowsAccountName
        {
            get
            {
                return ClaimsBuilder.GetClaimValue(this._identity, ClaimTypes.WindowsAccountName);
            }
        }

        /// <summary>
        /// Gets the unique identifier of the user.
        /// </summary>
        public int Id
        {
            get
            {
                return ClaimsBuilder.GetClaimValueInt(this._identity, ClaimTypes.NameIdentifier);
            }
        }

        /// <summary>
        /// Gets the name of the user.
        /// </summary>
        public string Name
        {
            get
            {
                return ClaimsBuilder.GetClaimValue(this._identity, ClaimTypes.Name);
            }
        }

        /// <summary>
        /// Gets the culture of the user.
        /// </summary>
        public string Culture
        {
            get
            {
                return ClaimsBuilder.GetClaimValue(this._identity, CustomClaimTypes.Culture);
            }
        }

        /// <summary>
        /// Gets the languages of the user.
        /// </summary>
        public IEnumerable<int> LanguageIds
        {
            get
            {
                return ClaimsBuilder.GetClaimValueIEnumerableInt(this._identity, CustomClaimTypes.UserLanguages);
            }
        }

        /// <summary>
        /// Gets the client website base url
        /// </summary>
        public string ClientWebsite
        {
            get
            {
                return ClaimsBuilder.GetClaimValue(this._identity, CustomClaimTypes.ClientWebsiteUrl);
            }
        }

        /// <summary>
        /// Gets the capabilities of the user.
        /// </summary>
        public IEnumerable<int> CapabilitiesIds
        {
            get
            {
                return ClaimsBuilder.GetClaimValueIEnumerableInt(this._identity, CustomClaimTypes.Capabilities);
            }
        }        
    }
}