﻿using Company.Infrastructure.Server;
using Company.Infrastructure.ServiceModel.Hosts;
using System;
using System.Diagnostics;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Text;
using System.Linq;

namespace Company.Hosts.Windows
{
    class Program
    {
        [STAThread]
        static void Main(string[] args)
        {
            if (Environment.UserInteractive)
            {
                HostHelper.Initialized += HostHelper_Initialized;
                HostHelper.Launching += HostHelper_Launching;
            }

            HostHelper.Start(delegate { });
        }

        #region Event Handlers

        static void HostHelper_Launching(object sender, EventArgs e)
        {
            foreach (ServiceHost host in HostHelper.Hosts)
            {
                WriteToEventLog(EventLogEntryType.SuccessAudit, string.Format("Service '{0}' hosted.", host.Description.Name), host.Description.Name);

                StringBuilder hostBehaviourDetails = ServiceHostManager.ReadHostBehaviours(host);
                Console.WriteLine(hostBehaviourDetails.ToString());

                // Trace: Host behavior details
                Trace.TraceInformation(hostBehaviourDetails.ToString());

                // Trace: Service namespace
                Trace.TraceInformation(string.Format("Namespace : {0}", host.Description.Namespace));

                // Each service endpoint
                foreach (ServiceEndpoint endPoint in host.Description.Endpoints)
                {
                    string address = endPoint.Address.ToString();

                    StringBuilder endpointDetails = new StringBuilder();

                    endpointDetails.AppendLine(string.Format("Endpoint Name: {0}", endPoint.Name));
                    endpointDetails.AppendLine(string.Format("Address: {0}", address));
                    endpointDetails.AppendLine(string.Format("Binding: {0} ({1})", endPoint.Binding.Name, endPoint.Binding.Scheme));
                    endpointDetails.AppendLine(string.Format("Contract: {0}", endPoint.Contract.Name));

                    // Read the specific binding details
                    ServiceHostManager.ReadBindingSecurityConfiguration(endPoint, endpointDetails);

                    // Console: Service endpoint
                    Console.WriteLine(endpointDetails.ToString());

                    // Trace: Service endpoint
                    Trace.TraceInformation(endpointDetails.ToString());
                }
            }

            // Display service types that could not be hosted
            if (HostHelper.FailedToHost.Any())
            {
                Console.WriteLine("This following services could not be hosted:");
                foreach (Type serviceType in HostHelper.FailedToHost)
                {
                    Console.WriteLine(serviceType.FullName);
                }
            }

            Console.WriteLine("Press Enter to quit");
            Console.ReadLine();
            HostHelper.End();
        }

        static void HostHelper_Initialized(object sender, EventArgs e)
        {
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Wrapper method use to write to the event log.
        /// </summary>
        /// <param name="logType"></param>
        /// <param name="logEntry"></param>
        private static void WriteToEventLog(EventLogEntryType logType, string logEntry, string hostName)
        {
            Trace.TraceInformation(logEntry);
            try
            {
                EventLog log = new EventLog();
                log.Source = hostName;
                log.WriteEntry(logEntry, logType);
            }
            catch (Exception)
            {
                // Swallow the exception.   
            }

            switch (logType)
            {
                case EventLogEntryType.Warning:
                    Console.ForegroundColor = ConsoleColor.Yellow;
                    break;

                case EventLogEntryType.Error:
                    Console.ForegroundColor = ConsoleColor.Red;
                    break;

                case EventLogEntryType.SuccessAudit:
                    Console.ForegroundColor = ConsoleColor.Green;
                    break;

                default:
                    Console.ForegroundColor = ConsoleColor.White;
                    break;
            }

            Console.WriteLine(string.Format("{0} - {1}", logType.ToString(), logEntry));
            Console.ForegroundColor = ConsoleColor.White;
        }

        #endregion
    }
}